/* Copyright 2010 ESRI
* 
* All rights reserved under the copyright laws of the United States
* and applicable international laws, treaties, and conventions.
* 
* You may freely redistribute and use this sample code, with or
* without modification, provided you include the original copyright
* notice and use restrictions.
* 
* See the use restrictions at http://help.arcgis.com/en/sdk/10.0/usageRestrictions.htm
* 
*/
package arcgissamples.loadsample;

import java.io.File;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;

import arcgissamples.display.CompassApplication;

public class Main{
	
    public static void main(String[] args)throws Exception {
        bootstrapArcobjectsJar();
        CompassApplication.main(args);
    }

    public static void bootstrapArcobjectsJar(){
    	//Get the ArcGIS Engine runtime, if it is available
    	String arcObjectsHome = System.getenv("AGSENGINEJAVA");
    	
    	//If the ArcGIS Engine runtime is not available, then we can try ArcGIS Desktop runtime
    	if(arcObjectsHome == null){
    		arcObjectsHome = System.getenv("AGSDESKTOPJAVA");
    	}
    	
    	//If no runtime is available, exit application gracefully
    	if(arcObjectsHome == null){
    		if(System.getProperty("os.name").toLowerCase().indexOf("win") > -1){
    			System.err.println("You must have ArcGIS Engine Runtime or ArcGIS Desktop " + 
    					"installed in order to execute this sample.");
    			System.err.println("Install one of the products above, then re-run this sample.");
    			System.err.println("Exiting execution of this sample...");
    			System.exit(0);	
    		}else{
    			System.err.println("You must have ArcGIS Engine Runtime installed " + 
    					"in order to execute this sample.");
    			System.err.println("Install the product above, then re-run this sample.");
    			System.err.println("Exiting execution of this sample...");
    			System.exit(0);	
    		}
    	}
    	
    	//Get the ArcGIS DeveloperKit for Java home location
    	String devKitHome = System.getenv("AGSDEVKITJAVA");
    	
    	//If no developer kit is available, exit the application gracefully
    	if(devKitHome == null){
    		System.err.println("You must have the ArcObjects Java SDK " + 
    					"installed in order to execute this sample.");
    		System.err.println("Please install the SDK, then re-run this sample.");
    		System.err.println("Exiting execution of this sample...");
    		System.exit(0);	
    	}
    	
    	//Get relative path to Java OpenGL native libraries
        String nativeLibrary = null;
        
        //Windows native libraries
        if (System.getProperty("os.name").toLowerCase().indexOf("win") > -1) {
			nativeLibrary = devKitHome + "java" + File.separator + "samples" + File.separator  
										 + "arcobjects" + File.separator + "lib"  + File.separator 
										 + "jogl" + File.separator + "jogl-1.1.0-windows-i586" + File.separator 
										 + "lib" + File.separator + "jogl.dll";
		//Solaris native libraries
		} else if (System.getProperty("os.name").toLowerCase().indexOf("sol") > -1) {
			nativeLibrary = devKitHome + "java" + File.separator + "samples" + File.separator 
									     + "arcobjects" + File.separator + "lib"  + File.separator 
									     + "jogl" + File.separator + "jogl-1.1.0-solaris-sparc" + File.separator
									     + "lib" + File.separator + "libjogl.so";
		//Linux native libraries
		} else if (System.getProperty("os.name").toLowerCase().indexOf("lin") > -1) {
			nativeLibrary = devKitHome + "java" + File.separator + "samples" + File.separator
									    + "arcobjects" + File.separator + "lib"  + File.separator 
									    + "jogl" + File.separator + "jogl-1.1.0-linux-i586" + File.separator 
									    + "lib" + File.separator + "libjogl.so";
		//Unknown platform
		}else{
			nativeLibrary = null;
		}
        

        if(nativeLibrary == null){
        	System.err.println("You are not using a supported operating system to execute this sample.");
        	System.err.println("Please install the ArcGIS software on an ESRI supported platform.");
        	System.err.println("Exiting execution of this sample...");
        	System.exit(0);	
        }
        
        //Load open gl native library
        System.load(nativeLibrary);
        
    	//Obtain the relative path to the arcobjects.jar file
        String jarPath = arcObjectsHome + "java" + File.separator + "lib" +
            			 File.separator + "arcobjects.jar";
        
        //Obtain the relative paths to jogl.jar and gluegen-rt.jar files
        String joglPath = devKitHome + "java" + File.separator + "samples" + File.separator 
        							 + "arcobjects" + File.separator + "lib" + File.separator 
        							 + "jogl" + File.separator + "jogl.jar";
        
        String gluegenPath = devKitHome + "java" + File.separator + "samples" + File.separator 
        								+ "arcobjects" + File.separator + "lib" + File.separator 
        								+ "jogl" + File.separator + "gluegen-rt.jar";

        //Create a new file
        File jarFile = new File(jarPath);
        File joglFile = new File(joglPath);
        File gluegenFile = new File(gluegenPath);
        
        //Test for file existence
        if(!jarFile.exists()){
        	System.err.println("The arcobjects.jar was not found in the following location: " +
        			jarFile.getParent());
        	System.err.println("Verify that arcobjects.jar can be located in the specified folder.");
        	System.err.println("If not present, try uninstalling your ArcGIS software and reinstalling it.");
        	System.err.println("Exiting execution of this sample...");
        	System.exit(0);
        }
        
        if(!joglFile.exists()){
        	System.err.println("The jogl.jar was not found in the following location: " +
        			joglFile.getParent());
        	System.err.println("Verify that jogl.jar can be located in the following folder: " +
        			joglFile.getParent());
        	System.err.println("If not present, try uninstalling your ArcGIS software and reinstalling it.");
        	System.err.println("Exiting execution of this sample...");
        	System.exit(0);
        }
        
        if(!gluegenFile.exists()){
        	System.err.println("The gluegen-rt.jar was not found in the following location: " +
        			gluegenFile.getParent());
        	System.err.println("Verify that gluegent-rt.jar can be located in the following folder: " +
        			gluegenFile.getParent());
        	System.err.println("If not present, try uninstalling your ArcGIS software and reinstalling it.");
        	System.err.println("Exiting execution of this sample...");
        	System.exit(0);
        }

        //Helps load classes and resources from a search path of URLs
        URLClassLoader sysloader = (URLClassLoader) ClassLoader.getSystemClassLoader();
        Class<URLClassLoader> sysclass = URLClassLoader.class;

        try{
        	Method method = sysclass.getDeclaredMethod("addURL", new Class[]{URL.class});
            method.setAccessible(true);
            method.invoke(sysloader, new Object[]{jarFile.toURI().toURL()});
            method.invoke(sysloader, new Object[]{joglFile.toURI().toURL()});
            method.invoke(sysloader, new Object[]{gluegenFile.toURI().toURL()});
        }catch (Throwable throwable){
            throwable.printStackTrace();
            System.err.println("Could not add arcobjects.jar to system classloader");
            System.err.println("Exiting execution of this sample...");
            System.exit(0);
        }
    }
}